@@ -293,10 +293,20 @@
 #'   example_train,
 #'   blueprint = default_formula_blueprint(intercept = TRUE)
 #' )
+#'
+#' # ---------------------------------------------------------------------------
+#' # Matrix output for predictors
+#'
+#' # You can change the `composition` of the predictor data set
+#' bp <- default_formula_blueprint(composition = "dgCMatrix")
+#' processed <- mold(log(num_1) ~ num_2 + fac_1, example_train, blueprint = bp)
+#' class(processed$predictors)
+#'
 #' @export
 default_formula_blueprint <- function(intercept = FALSE,
                                       allow_novel_levels = FALSE,
-                                      indicators = "traditional") {
+                                      indicators = "traditional",
+                                      composition = "tibble") {
   mold <- get_mold_formula_default_function_set()
   forge <- get_forge_formula_default_function_set()
 
@@ -305,7 +315,8 @@ default_formula_blueprint <- function(intercept = FALSE,
     forge = forge,
     intercept = intercept,
     allow_novel_levels = allow_novel_levels,
-    indicators = indicators
+    indicators = indicators,
+    composition = composition
   )
 
 }
@@ -323,6 +334,7 @@ new_default_formula_blueprint <- function(mold,
                                           ptypes = NULL,
                                           formula = NULL,
                                           indicators = "traditional",
+                                          composition = "tibble",
                                           terms = list(
                                             predictors = NULL,
                                             outcomes = NULL
@@ -340,6 +352,7 @@ new_default_formula_blueprint <- function(mold,
     ptypes = ptypes,
     formula = formula,
     indicators = indicators,
+    composition = composition,
     terms = terms,
     ...,
     subclass = c(subclass, "default_formula_blueprint")
@@ -441,6 +454,8 @@ mold_formula_default_process_predictors <- function(blueprint, data) {
 
   terms <- simplify_terms(framed$terms)
 
+  predictors <- recompose(predictors, blueprint$composition)
+
   blueprint_terms <- blueprint$terms
   blueprint_terms$predictors <- terms
   blueprint <- update_blueprint(blueprint, terms = blueprint_terms)
@@ -564,6 +579,8 @@ forge_formula_default_process_predictors <- function(blueprint, predictors) {
     data <- reattach_factorish_columns(data, predictors, factorish_names)
   }
 
+  data <- recompose(data, blueprint$composition)
+
   .offset <- extract_offset(framed$terms, framed$data)
 
   predictors_lst <- out$forge$process_terms_lst(
